<?php
/**
 * $Horde: imp/pgp.php,v 2.52 2003/01/16 08:50:45 jan Exp $
 *
 * Copyright 2002-2003 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

function _printKeyInfo($key = '')
{
    global $imp_pgp;
    $msg = '';

    $packet_info = $imp_pgp->pgpPacketInformation($key);

    if (empty($packet_info)) {
        _textWindowOutput('PGP Key Information', _("Invalid key"));
    } else {
        foreach (array_keys($packet_info['signature']) as $uid_idx) {
            if ($uid_idx == '_SIGNATURE') continue;
            $key_info = $imp_pgp->pgpPacketSignatureByUidIndex($key, $uid_idx);
            $msg .= _("Name") . ":             " . stripcslashes($key_info['name']) . "\n";
            $msg .= _("Key Type") . ":         " . (($key_info['key_type'] == 'public_key') ? _("Public Key") : _("Private Key")) . "\n";
            $msg .= _("Key Creation") . ":     " . strftime("%D", $key_info['key_created']) . "\n";
            $msg .= _("Expiration Date") . ":  " . (empty($key_info['key_expires']) ? '[' . _("Never") . ']' : strftime("%D", $key_info['key_expires'])) . "\n";
            $msg .= _("Key Length") . ":       " . $key_info['key_size'] . " Bytes\n";
            $msg .= _("Comment") . ":          " . (empty($key_info['comment']) ? '[' . _("None") . ']' : $key_info['comment']) . "\n";
            $msg .= _("E-Mail") . ":           " . $key_info['email'] . "\n";
            $msg .= _("Hash-Algorithm") . ":   " . (empty($key_info['micalg']) ? '[' . _("Unknown") . ']' : $key_info['micalg']) . "\n";
            $msg .= _("Key Fingerprint") . ":  " . (empty($key_info['fingerprint']) ? '[' . _("Unknown") . ']' : $key_info['fingerprint']) . "\n\n";
        }
        _textWindowOutput('PGP Key Information', $msg);
    }

    return;
}

function _outputPassphraseDialog()
{
    global $notification, $prefs, $registry, $secure_check, $selfURL;

    if (is_a($secure_check, 'PEAR_Error')) {
        $notification->push($secure_check->getMessage(), 'horde.warning');
    }

    $title = _("PGP Passphrase Input");
    $js_onLoad = 'document.passphrase_dialog.passphrase.focus();';
    require IMP_TEMPLATES . '/common-header.inc';
    $submit_url = Horde::addParameter($selfURL, 'actionID', PGP_PROCESS_PASSPHRASE_DIALOG);
    require IMP_BASE . '/status.php';
    require IMP_TEMPLATES . '/pgp/passphrase.inc';
}

function _importKeyDialog($target)
{
    global $actionID, $notification, $prefs, $registry, $selfURL;

    $title = _("Import PGP Key");
    require IMP_TEMPLATES . '/common-header.inc';
    require IMP_BASE . '/status.php';
    require IMP_TEMPLATES . '/pgp/import_key.inc';
}

function _actionWindow()
{
    $oid = Horde::getFormData('passphrase_action');
    require_once HORDE_BASE . '/lib/Cache.php';
    $cacheSess = &Cache::singleton('session');
    $cacheSess->setPruneFlag($oid, true);
    Horde::closeWindowJS($cacheSess->query($oid));
}

function _reloadWindow()
{
    Horde::closeWindowJS('opener.focus();opener.location.href="' . Horde::getFormData('reload') . '";');
}

function _getImportKey()
{
    global $notification;

    $key = Horde::getFormData('import_key');
    if (!empty($key)) return $key;

    include_once HORDE_BASE . '/lib/Server.php';
    $res = Server::wasFileUploaded('upload_key', _("key"));
    if (!is_a($res, 'PEAR_Error')) {
        foreach (file($_FILES['upload_key']['tmp_name']) as $value) {
            $temp_array[] = rtrim($value);
        }
        return implode("\n", $temp_array);
    } else {
        $notification->push($res->getMessage(), 'horde.error');
        return;
    }
}

function _textWindowOutput($filename, $msg)
{
    global $browser;

    $browser->downloadHeaders($filename, 'text/plain; charset=' . NLS::getCharset(), true, strlen($msg));
    echo $msg;
}


define('IMP_BASE', dirname(__FILE__));
require_once IMP_BASE . '/lib/base.php';
require_once IMP_BASE . '/lib/PGP.php';

$imp_pgp = new IMP_PGP();
$secure_check = $imp_pgp->requireSecureConnection();
$selfURL = Horde::applicationUrl('pgp.php');

/* Run through the action handlers */
$actionID = Horde::getFormData('actionID');
switch ($actionID) {

case PGP_GENERATE_KEY:
    /* Check that fields are filled out (except for Comment) and that the
       passphrases match. */
    $realname = Horde::getFormData('generate_realname');
    $email = Horde::getFormData('generate_email');
    $comment = Horde::getFormData('generate_comment');
    $keylength = Horde::getFormData('generate_keylength');
    $passphrase1 = Horde::getFormData('generate_passphrase1');
    $passphrase2 = Horde::getFormData('generate_passphrase2');

    if (empty($realname) || empty($email)) {
        $notification->push(_("Name and/or email can not be empty"), 'horde.error');
    } elseif (empty($passphrase1) || empty($passphrase2)) {
        $notification->push(_("Passphrases can not be empty"), 'horde.error');
    } elseif ($passphrase1 !== $passphrase2) {
        $notification->push(_("Passphrases do not match"), 'horde.error');
    } else {
        $result = $imp_pgp->generatePersonalKeys($realname, $email, $passphrase1, $comment, $keylength);
        if (is_a($result, 'PEAR_Error')) {
            $notification->push($result->getMessage(), $result->getCode());
        } else {
            $notification->push(_("Personal PGP keypair generated successfully."), 'horde.success');
        }
    }
    break;

case PGP_DELETE_KEY:
    $imp_pgp->deletePersonalKeys();
    $notification->push(_("Personal PGP keys deleted successfully."), 'horde.success');
    break;

case PGP_IMPORT_PUBLIC_KEY:
    _importKeyDialog(PGP_PROCESS_IMPORT_PUBLIC_KEY);
    exit;

case PGP_PROCESS_IMPORT_PUBLIC_KEY:
    $publicKey = _getImportKey();
    if (empty($publicKey)) {
        $notification->push(_("No PGP public key imported."), 'horde.error');
        $actionID = PGP_IMPORT_PUBLIC_KEY;
        _importKeyDialog(PGP_PROCESS_IMPORT_PUBLIC_KEY);
    } else {
        /* Add the public key to the storage system. */
        $key_info = $imp_pgp->addPublicKey($publicKey);
        if (is_a($key_info, 'PEAR_Error')) {
            $notification->push(_("Invalid PGP Key."), 'horde.error');
            $actionID = PGP_IMPORT_PUBLIC_KEY;
            _importKeyDialog(PGP_PROCESS_IMPORT_PUBLIC_KEY);
        } else {
            foreach ($key_info['signature'] as $sig) {
                $notification->push(sprintf(_("PGP Public Key for \"%s (%s)\" was successfully added."), $sig['name'], $sig['email']), 'horde.success');
            }
            _reloadWindow();
        }
    }
    exit;

case PGP_IMPORT_PERSONAL_PUBLIC_KEY:
    _importKeyDialog(PGP_IMPORT_PERSONAL_PRIVATE_KEY);
    exit;

case PGP_IMPORT_PERSONAL_PRIVATE_KEY:
    /* Check the public key. */
    if (!($publicKey = _getImportKey())) {
        $notification->push(_("No personal PGP public key imported."), 'horde.error');
        $actionID = PGP_IMPORT_PERSONAL_PUBLIC_KEY;
        _importKeyDialog(PGP_IMPORT_PERSONAL_PRIVATE_KEY);
    } else {
        if (!($key_info = $imp_pgp->pgpPacketInformation($publicKey))) {
            $notification->push(_("Invalid personal PGP public key."), 'horde.error');
            $actionID = PGP_IMPORT_PERSONAL_PUBLIC_KEY;
            _importKeyDialog(PGP_IMPORT_PERSONAL_PRIVATE_KEY);
        } else {
            $imp_pgp->addPersonalPublicKey($publicKey);
            $notification->push(_("PGP public key successfully added."), 'horde.success');
            _importKeyDialog(PGP_PROCESS_IMPORT_PERSONAL_KEY);
        }
    }
    exit;

case PGP_PROCESS_IMPORT_PERSONAL_KEY:
    /* Check the private key. */
    if (!($privateKey = _getImportKey())) {
        $notification->push(_("No personal PGP private key imported."), 'horde.error');
        $actionID = PGP_IMPORT_PERSONAL_PRIVATE_KEY;
        _importKeyDialog(PGP_PROCESS_IMPORT_PERSONAL_KEY);
    } else {
        if (!($key_info = $imp_pgp->pgpPacketInformation($privateKey))) {
            $notification->push(_("Invalid personal PGP private key."), 'horde.error');
            $actionID = PGP_IMPORT_PERSONAL_PRIVATE_KEY;
            _importKeyDialog(PGP_PROCESS_IMPORT_PERSONAL_KEY);
        } else {
            $imp_pgp->addPersonalPrivateKey($privateKey);
            $notification->push(_("PGP private key successfully added."), 'horde.success');
            _reloadWindow();
        }
    }
    exit;

case PGP_VIEW_PUBLIC_KEY:
    $key = $imp_pgp->getPublicKey(Horde::getFormData('email'));
    if (is_a($key, 'PEAR_Error')) {
        $key = $key->getMessage();
    }
    _textWindowOutput('PGP Public Key', $key);
    exit;

case PGP_VIEW_PERSONAL_PUBLIC_KEY:
    _textWindowOutput('PGP Personal Public Key', $imp_pgp->getPersonalPublicKey());
    exit;

case PGP_INFO_PUBLIC_KEY:
    $key = $imp_pgp->getPublicKey(Horde::getFormData('email'));
    if (is_a($key, 'PEAR_Error')) {
        $key = $key->getMessage();
    }
    _printKeyInfo($key);
    exit;

case PGP_INFO_PERSONAL_PUBLIC_KEY:
    _printKeyInfo($imp_pgp->getPersonalPublicKey());
    exit;

case PGP_VIEW_PERSONAL_PRIVATE_KEY:
    _textWindowOutput('PGP Personal Private Key', $imp_pgp->getPersonalPrivateKey());
    exit;

case PGP_INFO_PERSONAL_PRIVATE_KEY:
    _printKeyInfo($imp_pgp->getPersonalPrivateKey());
    exit;

case PGP_DELETE_PUBLIC_KEY:
    $result = $imp_pgp->deletePublicKey(Horde::getFormData('email'));
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result->getMessage(), $result->getCode());
    } else {
        $notification->push(sprintf(_("PGP Public Key for \"%s\" was successfully deleted."), Horde::getFormData('email')), 'horde.success');
    }
    break;

case PGP_ENABLE:
    $prefs->setValue('use_pgp', Horde::getFormData('use_pgp'));
    break;

case PGP_SAVE_OPTIONS:
    $prefs->setValue('use_pgp', Horde::getFormData('use_pgp'));
    $prefs->setValue('pgp_attach_pubkey', Horde::getFormData('pgp_attach_pubkey'));
    $prefs->setValue('pgp_auto_sign', Horde::getFormData('pgp_auto_sign'));
    $prefs->setValue('pgp_scan_body', Horde::getFormData('pgp_scan_body'));
    $notification->push(_("Preferences successfully updated."), 'horde.success');
    break;

case PGP_SAVE_ATTACHMENT_PUBLIC_KEY:
    include_once HORDE_BASE . '/lib/Cache.php';
    include_once HORDE_BASE . '/lib/MIME/Part.php';

    /* Retrieve the key from the cache. */
    $cache = &Cache::singleton('session');
    $mime_part = $cache->query(Horde::getFormData('mimecache'));
    $mime_part->transferDecodeContents();

    /* Add the public key to the storage system. */
    $key_info = $imp_pgp->addPublicKey($mime_part->getContents());
    if (is_a($key_info, 'PEAR_Error')) {
        $notification->push($key_info->getMessage(), $key_info->getCode());
    } else {
        Horde::closeWindowJS();
    }
    exit;

case PGP_OPEN_PASSPHRASE_DIALOG:
    if ($imp_pgp->getPassphrase()) {
        Horde::closeWindowJS();
    } else {
        _outputPassphraseDialog();
    }
    exit;

case PGP_PROCESS_PASSPHRASE_DIALOG:
    if (is_a($secure_check, 'PEAR_Error')) {
        _outputPassphraseDialog();
    } elseif (Horde::getFormData('passphrase')) {
        if ($imp_pgp->storePassphrase(Horde::getFormData('passphrase'))) {
            if (Horde::getFormData('passphrase_action')) {
                _actionWindow();
            } elseif (Horde::getFormData('reload')) {
                _reloadWindow();
            } else {
                Horde::closeWindowJS();
            }
        } else {
            $notification->push("Invalid passphrase entered.", 'horde.error');
            _outputPassphraseDialog();
        }
    } else {
        $notification->push("No passphrase entered.", 'horde.error');
        _outputPassphraseDialog();
    }
    exit;

case PGP_UNSET_PASSPHRASE:
    $imp_pgp->unsetPassphrase();
    $notification->push(_("Passphrase successfully unloaded."), 'horde.success');
    break;

case PGP_SEND_PUBLIC_KEY:
    $result = $imp_pgp->sendToPublicKeyserver($imp_pgp->getPersonalPublicKey());
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result->getMessage(), $result->getCode());
    } else {
        $notification->push(_("Key successfully sent to the public keyserver."), 'horde.success');
    }
    break;

}

/* Get list of Public Keys on keyring. */
$pubkey_list = $imp_pgp->listPublicKeys();
if (is_a($pubkey_list, 'PEAR_Error')) {
    $notification->push($pubkey_list->getMessage(), $pubkey_list->getCode());
}

/* Get passphrase (if available). */
$passphrase = $imp_pgp->getPassphrase();

$title = sprintf(_("%s PGP Administration"), _("Mail"));
$js_onLoad = null;
require IMP_TEMPLATES . '/common-header.inc';
require IMP_BASE . '/menu.php';
require IMP_BASE . '/status.php';

/* If PGP preference not active, do NOT show PGP Admin screen. */
if ($prefs->getValue('use_pgp')) {
    $openpgpwin = $imp_pgp->getJSOpenWinCode(PGP_OPEN_PASSPHRASE_DIALOG);
    require_once IMP_TEMPLATES . '/pgp/open_pgp_win.js';
    require_once IMP_TEMPLATES . '/pgp/open_pgp_import.js';
    require IMP_TEMPLATES . '/pgp/pgp.inc';
} else {
    require IMP_TEMPLATES . '/pgp/notactive.inc';
}

require IMP_TEMPLATES . '/common-footer.inc';
